/*============================================================================
	File:		099 - A50 - TEMPDB - Allocation Contention.sql

	Summary:	This script demonstrates the different runtimes
				the problem with allocation contention in a wrong
				configured TEMPDB database

				THIS SCRIPT IS PART OF THE TRACK: "Analysis of SQL Server"

	Date:		September 2017

	SQL Server Version: 2008 / 2012 / 2014 / 2016 / 2017
------------------------------------------------------------------------------
	Written by Uwe Ricken, db Berater GmbH

	This script is intended only as a supplement to demos and lectures
	given by Uwe Ricken.  
  
	THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
	ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
	TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
	PARTICULAR PURPOSE.
============================================================================*/
USE master;
GO

-- Preparation 1:	Drop an existing demo database
IF DB_ID(N'demo_db') IS NOT NULL
BEGIN
	ALTER DATABASE [demo_db] SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE [demo_db];
END
GO

-- Preparation 2:	Reduce TEMPDB to 1 data file
USE tempdb;
GO

-- Copy the output into a new window and execute it
-- if some files cannot be deleted restart the instance
-- and try it again!
SELECT	N'DBCC SHRINKFILE(' + QUOTENAME(name) + N', EMPTYFILE);
ALTER DATABASE [tempdb] REMOVE FILE ' + QUOTENAME(name) + N';'
FROM	sys.master_files
WHERE	database_id = 2
		AND type = 0
		AND file_id > 1;
GO

-- Preparation 3:	Create the demo database with a demo table
CREATE DATABASE demo_db;
GO

USE demo_db;
GO

SELECT	TOP (2000)
		message_id,
		CAST([text] AS CHAR(2000)) AS [text]
INTO	dbo.messages
FROM	sys.messages
WHERE	language_id = 1031;
GO

CREATE UNIQUE CLUSTERED INDEX cuix_messages
ON dbo.messages (message_id);
GO

-- Preparation 4:	Example proc for workload
IF OBJECT_ID(N'dbo.proc_fill_tempdata', N'P') IS NOT NULL
	DROP PROC dbo.proc_fill_tempdata;
	GO

CREATE OR ALTER PROC dbo.proc_fill_tempdata
	@UseTableVariable BIT,
	@Duration	TINYINT = 120
AS
BEGIN
	SET NOCOUNT ON;
	DECLARE	@EndTime	DATETIME2(0) = DATEADD(MINUTE, @Duration, GETDATE());

	WHILE (GETDATE() <= @EndTime)
	BEGIN
		IF @UseTableVariable = 1
		BEGIN
			DECLARE	@TV AS TABLE
			(
				message_id INT,
				C1 CHAR(2000)
			);

			INSERT INTO @TV (message_id, C1)
			SELECT TOP (16) message_id, [text]
			FROM	sys.messages;
		END
		ELSE
		BEGIN
			CREATE TABLE #T
			(
				message_id INT,
				[C1] CHAR(2000)
			);

			INSERT INTO #T (message_id, C1)
			SELECT message_id, [text]
			FROM	sys.messages;
		END
	END

	SET NOCOUNT OFF;
END
GO

-- Now run the stored procedure for 3 minutes each with the following command
-- C:\Program Files\Microsoft Corporation\RMLUtils\ostress -SNB-LENOVO-I\SQL_2016 -E -Q"EXEC dbo.proc_fill_tempdata @UseTableVariable = 1, @Duration = 3;" -ddemo_db -q -n50
-- C:\Program Files\Microsoft Corporation\RMLUtils\ostress -SNB-LENOVO-I\SQL_2016 -E -Q"EXEC dbo.proc_fill_tempdata @UseTableVariable = 0, @Duration = 3;" -ddemo_db -q -n50

-- While the demo is running check the waiting tasks and
-- its resources!
WITH Tasks
AS
(
	SELECT	session_id,
			wait_type,
			wait_duration_ms,
			blocking_session_id,
			resource_description,
			PageID = CAST
			(
				RIGHT
				(
						resource_description,
						LEN(resource_description) - CHARINDEX(':', resource_description, 3)
				) AS INT
			)
	FROM	sys.dm_os_waiting_tasks AS DOWT WITH (NOLOCK)
	WHERE	wait_type LIKE 'PAGE%LATCH_%' AND
			resource_description LIKE '2:%'
)
SELECT	session_id,
		wait_type,
		wait_duration_ms,
		blocking_session_id,
		resource_description,
		ResourceType =	CASE WHEN PageID = 1 Or PageID % 8088 = 0 THEN 'PFS'
							 WHEN PageID = 2 Or PageID % 511232 = 0 THEN 'GAM'
							 WHEN PageID = 3 Or (PageID - 1) % 511232 = 0 THEN 'SGAM'
							 ELSE 'OTHER'
						END
FROM Tasks;
GO

-- After the demo we add as much files as we have cores in the system!
-- Before adding new tempdb files we reduce the size of the first file
-- to 1024 MB
USE tempdb;
GO

DBCC SHRINKFILE ('tempdev', 1024);
GO

DECLARE	@stmt		NVARCHAR(1024);
DECLARE	@FilePath	NVARCHAR(260);
DECLARE	@MaxFiles	TINYINT;
DECLARE	@Counter	TINYINT = 2;

SELECT	@MaxFiles = cpu_count
FROM sys.dm_os_sys_info;

SELECT	@FilePath = REVERSE(RIGHT(REVERSE(filename), LEN(filename) - CHARINDEX('\', REVERSE(filename))))
FROM sys.sysfiles
WHERE	FileId = 1;

IF @MaxFiles > 4
	SET @MaxFiles = 4;

WHILE @Counter <= @MaxFiles
	  OR @Counter <= 4
BEGIN
	SET	@stmt = 'ALTER DATABASE [tempdb]
ADD FILE
(
	Name = ''tempdev_' + RIGHT(N'0' + CAST(@Counter AS NVARCHAR(2)), 2) + ''',
	FILENAME = ''' + @FilePath + N'\tempdev_' +RIGHT(N'0' + CAST(@Counter AS NVARCHAR(2)), 2) + N'.ndf'',
	SIZE = 1024,
	MAXSIZE = 2048
);
GO';

	PRINT @stmt;
	SET @Counter += 1;
END

-- Now run the stored procedure for 3 minutes each with the following command
-- C:\Program Files\Microsoft Corporation\RMLUtils\ostress -SNB-LENOVO-I\SQL_2016 -E -Q"EXEC dbo.proc_fill_tempdata @UseTableVariable = 1, @Duration = 3;" -ddemo_db -q -n50
-- C:\Program Files\Microsoft Corporation\RMLUtils\ostress -SNB-LENOVO-I\SQL_2016 -E -Q"EXEC dbo.proc_fill_tempdata @UseTableVariable = 0, @Duration = 3;" -ddemo_db -q -n50
WITH Tasks
AS
(
	SELECT	session_id,
			wait_type,
			wait_duration_ms,
			blocking_session_id,
			resource_description,
			PageID = CAST
			(
				RIGHT
				(
						resource_description,
						LEN(resource_description) - CHARINDEX(':', resource_description, 3)
				) AS INT
			)
	FROM	sys.dm_os_waiting_tasks AS DOWT WITH (NOLOCK)
	WHERE	wait_type LIKE 'PAGE%LATCH_%' AND
			resource_description LIKE '2:%'
)
SELECT	session_id,
		wait_type,
		wait_duration_ms,
		blocking_session_id,
		resource_description,
		ResourceType =	CASE WHEN PageID = 1 Or PageID % 8088 = 0 THEN 'PFS'
							 WHEN PageID = 2 Or PageID % 511232 = 0 THEN 'GAM'
							 WHEN PageID = 3 Or (PageID - 1) % 511232 = 0 THEN 'SGAM'
							 ELSE 'OTHER'
						END
FROM Tasks;
GO
